=== modified file 'NEWS'
--- NEWS	2009-03-10 06:07:38 +0000
+++ NEWS	2009-03-10 06:46:19 +0000
@@ -70,6 +70,9 @@
       batches up requests into approx 5MB requests, rather than a separate
       request for each file. (John Arbash Meinel)
 
+    * Progress bars now show the rate of network activity for
+      ``bzr+ssh://`` and ``bzr://`` connections.  (Andrew Bennetts)
+
     * Support for GSSAPI authentication when using HTTP or HTTPS. 
       (Jelmer Vernooij)
 

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2009-02-23 15:29:35 +0000
+++ bzrlib/osutils.py	2009-03-10 05:11:47 +0000
@@ -1623,15 +1623,21 @@
     return b
 
 
-def send_all(socket, bytes):
+def send_all(socket, bytes, report_activity=None):
     """Send all bytes on a socket.
 
     Regular socket.sendall() can give socket error 10053 on Windows.  This
     implementation sends no more than 64k at a time, which avoids this problem.
+
+    :param report_activity: Call this as bytes are read, see
+        Transport._report_activity
     """
     chunk_size = 2**16
     for pos in xrange(0, len(bytes), chunk_size):
-        until_no_eintr(socket.sendall, bytes[pos:pos+chunk_size])
+        block = bytes[pos:pos+chunk_size]
+        if report_activity is not None:
+            report_activity(len(block), 'write')
+        until_no_eintr(socket.sendall, block)
 
 
 def dereference_path(path):

=== modified file 'bzrlib/smart/medium.py'
--- bzrlib/smart/medium.py	2009-03-06 08:17:36 +0000
+++ bzrlib/smart/medium.py	2009-03-10 04:57:28 +0000
@@ -40,6 +40,7 @@
     osutils,
     symbol_versioning,
     trace,
+    ui,
     urlutils,
     )
 from bzrlib.smart import client, protocol
@@ -162,6 +163,18 @@
         self._push_back(excess)
         return line
 
+    def _report_activity(self, bytes, direction):
+        """Notify that this medium has activity.
+
+        Implementations should call this from all methods that actually do IO.
+        Be careful that it's not called twice, if one method is implemented on
+        top of another.
+
+        :param bytes: Number of bytes read or written.
+        :param direction: 'read' or 'write' or None.
+        """
+        ui.ui_factory.report_transport_activity(self, bytes, direction)
+
 
 class SmartServerStreamMedium(SmartMedium):
     """Handles smart commands coming over a stream.
@@ -274,7 +287,9 @@
     def _read_bytes(self, desired_count):
         # We ignore the desired_count because on sockets it's more efficient to
         # read large chunks (of _MAX_READ_SIZE bytes) at a time.
-        return osutils.until_no_eintr(self.socket.recv, _MAX_READ_SIZE)
+        bytes = osutils.until_no_eintr(self.socket.recv, _MAX_READ_SIZE)
+        self._report_activity(len(bytes), 'read')
+        return bytes
 
     def terminate_due_to_error(self):
         # TODO: This should log to a server log file, but no such thing
@@ -283,7 +298,7 @@
         self.finished = True
 
     def _write_out(self, bytes):
-        osutils.send_all(self.socket, bytes)
+        osutils.send_all(self.socket, bytes, self._report_activity)
 
 
 class SmartServerPipeStreamMedium(SmartServerStreamMedium):
@@ -689,6 +704,7 @@
     def _accept_bytes(self, bytes):
         """See SmartClientStreamMedium.accept_bytes."""
         self._writeable_pipe.write(bytes)
+        self._report_activity(len(bytes), 'write')
 
     def _flush(self):
         """See SmartClientStreamMedium._flush()."""
@@ -696,7 +712,9 @@
 
     def _read_bytes(self, count):
         """See SmartClientStreamMedium._read_bytes."""
-        return self._readable_pipe.read(count)
+        bytes = self._readable_pipe.read(count)
+        self._report_activity(len(bytes), 'read')
+        return bytes
 
 
 class SmartSSHClientMedium(SmartClientStreamMedium):
@@ -730,6 +748,7 @@
         """See SmartClientStreamMedium.accept_bytes."""
         self._ensure_connection()
         self._write_to.write(bytes)
+        self._report_activity(len(bytes), 'write')
 
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
@@ -765,7 +784,9 @@
         if not self._connected:
             raise errors.MediumNotConnected(self)
         bytes_to_read = min(count, _MAX_READ_SIZE)
-        return self._read_from.read(bytes_to_read)
+        bytes = self._read_from.read(bytes_to_read)
+        self._report_activity(len(bytes), 'read')
+        return bytes
 
 
 # Port 4155 is the default port for bzr://, registered with IANA.
@@ -787,7 +808,7 @@
     def _accept_bytes(self, bytes):
         """See SmartClientMedium.accept_bytes."""
         self._ensure_connection()
-        osutils.send_all(self._socket, bytes)
+        osutils.send_all(self._socket, bytes, self._report_activity)
 
     def disconnect(self):
         """See SmartClientMedium.disconnect()."""
@@ -850,13 +871,16 @@
         # We ignore the desired_count because on sockets it's more efficient to
         # read large chunks (of _MAX_READ_SIZE bytes) at a time.
         try:
-            return self._socket.recv(_MAX_READ_SIZE)
+            bytes = osutils.until_no_eintr(self._socket.recv, _MAX_READ_SIZE)
         except socket.error, e:
             if len(e.args) and e.args[0] == errno.ECONNRESET:
                 # Callers expect an empty string in that case
                 return ''
             else:
                 raise
+        else:
+            self._report_activity(len(bytes), 'read')
+            return bytes
 
 
 class SmartClientStreamMediumRequest(SmartClientMediumRequest):

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2009-02-23 15:29:35 +0000
+++ bzrlib/transport/http/__init__.py	2009-03-10 04:57:28 +0000
@@ -622,6 +622,14 @@
             raise errors.SmartProtocolError(str(e))
         return body_filelike
 
+    def _report_activity(self, bytes, direction):
+        """See SmartMedium._report_activity.
+
+        Does nothing; the underlying plain HTTP transport will report the
+        activity that this medium would report.
+        """
+        pass
+
 
 # TODO: May be better located in smart/medium.py with the other
 # SmartMediumRequest classes

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2009-02-26 04:25:00 +0000
+++ bzrlib/transport/remote.py	2009-03-10 04:57:28 +0000
@@ -136,6 +136,14 @@
         # No credentials
         return None, None
 
+    def _report_activity(self, bytes, direction):
+        """See Transport._report_activity.
+
+        Does nothing; the smart medium will report activity triggered by a
+        RemoteTransport.
+        """
+        pass
+
     def is_readonly(self):
         """Smart server transport can do read/write file operations."""
         try:

